@@ -28,6 +28,10 @@
 
 class QQuickItem;
 
+/**
+ * @class Units
+ * @short Expose sizes to QML
+ */
 class Units : public QObject
 {
     Q_OBJECT
@@ -96,10 +100,12 @@ class Units : public QObject
     Q_PROPERTY(int shortDuration READ shortDuration NOTIFY durationChanged)
 
 public:
+/// @cond INTERNAL_DOCS
+
     Units(QObject *parent = 0);
     ~Units();
 
-    bool eventFilter(QObject *watched, QEvent *event);
+    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
 
     /**
      * @return pixel value for a grid Unit. Depends on DPI and font size.
@@ -116,6 +122,14 @@ public:
      */
     QQmlPropertyMap *iconSizes() const;
 
+    /**
+     * @return a size rounded tothe nearest inferior standard icon size.
+     *           sizes larger than iconSizes.huge, it will be returned unmodified
+     * @param int size the size we want to be rounded down
+     * @see iconSizes
+     */
+    Q_INVOKABLE static int roundToIconSize(int size);
+
     /**
      * @return Pixel value for large spacing between elements.
      * @since 5.0
@@ -139,6 +153,7 @@ public:
      * @since 5.0
      */
     int shortDuration() const;
+/// @endcond
 
 Q_SIGNALS:
     void devicePixelRatioChanged();
@@ -149,18 +164,17 @@ Q_SIGNALS:
 
 private Q_SLOTS:
     void iconLoaderSettingsChanged();
-    void settingsFileChanged(const QString &settings);
+    void settingsFileChanged(const QString &file);
 
 private:
     void updateDevicePixelRatio();
     void updateSpacing();
+    void updatePlasmaRCSettings();
     /**
      * @return The dpi-adjusted size for a given icon size
      */
     int devicePixelIconSize(const int size) const;
 
-    Plasma::Theme m_theme;
-
     int m_gridUnit;
     qreal m_devicePixelRatio;
     qreal m_dpi;
