@@ -31,6 +31,8 @@
 #include <kplugininfo.h>
 #include <KNotification>
 
+#include <KPackage/Package>
+
 #include "plasma/applet.h"
 
 class KKeySequenceWidget;
@@ -79,6 +81,7 @@ public:
 
     // applet attributes
     Types::ImmutabilityType immutability;
+    Types::ImmutabilityType oldImmutability;
     QString launchErrorMessage;
 
     // applet info we keep around in case its needed
@@ -92,7 +95,7 @@ public:
 
     // sripting and package stuff
     AppletScript *script;
-    Package *package;
+    KPackage::Package *package;
     KConfigLoader *configLoader;
 
     // actions stuff; put activationAction into actions?
