@@ -1,5 +1,6 @@
 /*
  *   Copyright 2013 Marco Martin <mart@kde.org>
+ *   Copyright 2015 Eike Hein <hein@kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -38,6 +39,8 @@
 #include <Plasma/Corona>
 #include <Plasma/PluginLoader>
 
+namespace PlasmaQuick
+{
 
 //////////////////////////////ConfigModel
 
@@ -48,7 +51,7 @@ public:
     ~ConfigModelPrivate();
 
     ConfigModel *q;
-    QList<ConfigCategory*> categories;
+    QList<ConfigCategory *> categories;
     QWeakPointer<Plasma::Applet> appletInterface;
 
     void appendCategory(ConfigCategory *c);
@@ -149,10 +152,11 @@ QVariant ConfigModelPrivate::get(int row) const
     } else {
         value["source"] = categories.at(row)->source();
     }
+    value["visible"] = categories.at(row)->visible();
+
     return value;
 }
 
-
 ConfigModel::ConfigModel(QObject *parent)
     : QAbstractListModel(parent),
       d(new ConfigModelPrivate(this))
@@ -162,6 +166,7 @@ ConfigModel::ConfigModel(QObject *parent)
     roleNames[IconRole] = "icon";
     roleNames[SourceRole] = "source";
     roleNames[PluginNameRole] = "pluginName";
+    roleNames[VisibleRole] = "visible";
 
     setRoleNames(roleNames);
 }
@@ -179,7 +184,7 @@ int ConfigModel::rowCount(const QModelIndex &index) const
     return d->categories.count();
 }
 
-QVariant ConfigModel::data(const QModelIndex& index, int role) const
+QVariant ConfigModel::data(const QModelIndex &index, int role) const
 {
     if (index.row() < 0 || index.row() >= d->categories.count()) {
         return QVariant();
@@ -197,6 +202,8 @@ QVariant ConfigModel::data(const QModelIndex& index, int role) const
         }
     case PluginNameRole:
         return d->categories.at(index.row())->pluginName();
+    case VisibleRole:
+        return d->categories.at(index.row())->visible();
     default:
         return QVariant();
     }
@@ -218,6 +225,18 @@ void ConfigModel::appendCategory(const QString &iconName, const QString &name,
     d->appendCategory(cat);
 }
 
+void ConfigModel::appendCategory(const QString &iconName, const QString &name,
+                                 const QString &path, const QString &pluginName, bool visible)
+{
+    ConfigCategory *cat = new ConfigCategory(this);
+    cat->setIcon(iconName);
+    cat->setName(name);
+    cat->setSource(path);
+    cat->setPluginName(pluginName);
+    cat->setVisible(visible);
+    d->appendCategory(cat);
+}
+
 void ConfigModel::clear()
 {
     d->clear();
@@ -236,11 +255,12 @@ Plasma::Applet *ConfigModel::applet() const
 QQmlListProperty<ConfigCategory> ConfigModel::categories()
 {
     return QQmlListProperty<ConfigCategory>(this, 0, ConfigModelPrivate::categories_append,
-                                             ConfigModelPrivate::categories_count,
-                                             ConfigModelPrivate::categories_at,
-                                             ConfigModelPrivate::categories_clear);
+                                            ConfigModelPrivate::categories_count,
+                                            ConfigModelPrivate::categories_at,
+                                            ConfigModelPrivate::categories_clear);
 
 }
 
+}
 
 #include "moc_configmodel.cpp"
