@@ -59,6 +59,7 @@ public:
     DialogPrivate(Dialog *dialog)
         : q(dialog),
           location(Plasma::Types::BottomEdge),
+          frameSvgItem(0),
           type(Dialog::Normal),
           hideOnWindowDeactivate(false),
           outputOnly(false),
@@ -331,6 +332,9 @@ void DialogPrivate::updateMinimumWidth()
     //on the scene
     auto margin = frameSvgItem->fixedMargins();
     int minimumWidth = mainItemLayout->property("minimumWidth").toInt() + margin->left() + margin->right();
+    if (q->screen()) {
+        minimumWidth = qMin(q->screen()->availableGeometry().width(), minimumWidth);
+    }
     q->contentItem()->setWidth(qMax(q->width(), minimumWidth));
     q->setWidth(qMax(q->width(), minimumWidth));
 
@@ -354,6 +358,9 @@ void DialogPrivate::updateMinimumHeight()
     //on the scene
     auto margin = frameSvgItem->fixedMargins();
     int minimumHeight = mainItemLayout->property("minimumHeight").toInt() + margin->top() + margin->bottom();
+    if (q->screen()) {
+        minimumHeight = qMin(q->screen()->availableGeometry().height(), minimumHeight);
+    }
     q->contentItem()->setHeight(qMax(q->height(), minimumHeight));
     q->setHeight(qMax(q->height(), minimumHeight));
 
@@ -373,6 +380,9 @@ void DialogPrivate::updateMaximumWidth()
 
     auto margin = frameSvgItem->fixedMargins();
     int maximumWidth = mainItemLayout->property("maximumWidth").toInt() + margin->left() + margin->right();
+    if (q->screen()) {
+        maximumWidth = qMin(q->screen()->availableGeometry().width(), maximumWidth);
+    }
     q->contentItem()->setWidth(qMax(q->width(), maximumWidth));
     q->setWidth(qMax(q->width(), maximumWidth));
 
@@ -392,6 +402,9 @@ void DialogPrivate::updateMaximumHeight()
 
     auto margin = frameSvgItem->fixedMargins();
     int maximumHeight = mainItemLayout->property("maximumHeight").toInt() + margin->top() + margin->bottom();
+    if (q->screen()) {
+        maximumHeight = qMin(q->screen()->availableGeometry().height(), maximumHeight);
+    }
     q->contentItem()->setHeight(qMax(q->height(), maximumHeight));
     q->setHeight(qMin(q->height(), maximumHeight));
 
@@ -400,7 +413,7 @@ void DialogPrivate::updateMaximumHeight()
 
 void DialogPrivate::updateLayoutParameters()
 {
-    if (!componentComplete || !mainItem || !q->isVisible()) {
+    if (!componentComplete || !mainItem || !q->isVisible() || !mainItemLayout) {
         return;
     }
     Q_ASSERT(mainItem);
@@ -423,6 +436,12 @@ void DialogPrivate::updateLayoutParameters()
     minimumWidth += margin->left() + margin->right();
     maximumWidth += margin->left() + margin->right();
 
+    if (q->screen()) {
+        minimumWidth = qMin(q->screen()->availableGeometry().width(), minimumWidth);
+        minimumHeight = qMin(q->screen()->availableGeometry().height(), minimumHeight);
+        maximumWidth = qMin(q->screen()->availableGeometry().width(), maximumWidth);
+        maximumHeight = qMin(q->screen()->availableGeometry().height(), maximumHeight);
+    }
 
     const QSize finalSize(qBound(minimumWidth, q->width(), maximumWidth),
                           qBound(minimumHeight, q->height(), maximumHeight));
@@ -533,6 +552,9 @@ void DialogPrivate::syncToMainItemSize()
     if (!componentComplete || !q->isVisible()) {
         return;
     }
+    if (mainItem->width() <= 0 || mainItem->height() <= 0) {
+        qWarning() << "trying to show an empty dialog";
+    }
 
     updateTheme();
     if (visualParent) {
@@ -590,7 +612,7 @@ void DialogPrivate::slotWindowPositionChanged()
 {
     // Tooltips always have all the borders
     // floating windows have all borders
-    if ((q->flags() & Qt::ToolTip) || location == Plasma::Types::Floating) {
+    if (!q->isVisible() || (q->flags() & Qt::ToolTip) || location == Plasma::Types::Floating) {
         return;
     }
 
@@ -673,10 +695,17 @@ void Dialog::setMainItem(QQuickItem *mainItem)
 {
     if (d->mainItem != mainItem) {
         d->hintsCommitTimer.stop();
+
         if (d->mainItem) {
+            disconnect(d->mainItem, 0, this, 0);
             d->mainItem->setVisible(false);
         }
 
+        if (d->mainItemLayout) {
+            disconnect(d->mainItemLayout, 0, this, 0);
+        }
+
+
         d->mainItem = mainItem;
 
         if (mainItem) {
@@ -704,9 +733,7 @@ void Dialog::setMainItem(QQuickItem *mainItem)
                     layout = child;
                 }
             }
-            if (d->mainItemLayout) {
-                disconnect(d->mainItemLayout, 0, this, 0);
-            }
+
             d->mainItemLayout = layout;
 
             if (layout) {
@@ -802,19 +829,23 @@ QPoint Dialog::popupPosition(QQuickItem *item, const QSize &size)
     if (outsideParentWindow) {
         parentGeometryBounds = item->window()->geometry();
     } else {
-        parentGeometryBounds = QRect(pos.toPoint(), QSize(item->width(), item->height()));
+        parentGeometryBounds = item->mapRectToScene(item->boundingRect()).toRect();
+        if (item->window()) {
+            parentGeometryBounds.moveTopLeft(item->window()->mapToGlobal(parentGeometryBounds.topLeft()));
+            pos = parentGeometryBounds.topLeft();
+        }
     }
 
-    const QPoint topPoint(pos.x() + (item->boundingRect().width() - size.width()) / 2,
+    const QPoint topPoint(pos.x() + (item->mapRectToScene(item->boundingRect()).width() - size.width()) / 2,
                           parentGeometryBounds.top() - size.height());
-    const QPoint bottomPoint(pos.x() + (item->boundingRect().width() - size.width()) / 2,
+    const QPoint bottomPoint(pos.x() + (item->mapRectToScene(item->boundingRect()).width() - size.width()) / 2,
                              parentGeometryBounds.bottom());
 
     const QPoint leftPoint(parentGeometryBounds.left() - size.width(),
-                           pos.y() + (item->boundingRect().height() - size.height()) / 2);
+                           pos.y() + (item->mapRectToScene(item->boundingRect()).height() - size.height()) / 2);
 
     const QPoint rightPoint(parentGeometryBounds.right(),
-                            pos.y() + (item->boundingRect().height() - size.height()) / 2);
+                            pos.y() + (item->mapRectToScene(item->boundingRect()).height() - size.height()) / 2);
 
     QPoint dialogPos;
     if (d->location == Plasma::Types::TopEdge) {
@@ -926,7 +957,7 @@ void Dialog::resizeEvent(QResizeEvent* re)
     QQuickWindow::resizeEvent(re);
 
     //A dialog can be resized even if no mainItem has ever been set
-    if (!d->mainItem) {
+    if (!isVisible() || !d->mainItem) {
         return;
     }
 
@@ -1051,7 +1082,9 @@ bool Dialog::event(QEvent *event)
                                     d->positionAdjustedForMainItem(me->windowPos()) + position(),
                                     me->button(), me->buttons(), me->modifiers());
 
-                    QCoreApplication::sendEvent(this, &me2);
+                    if (isVisible()) {
+                        QCoreApplication::sendEvent(this, &me2);
+                    }
                     return true;
                 }
                 break;
@@ -1066,7 +1099,9 @@ bool Dialog::event(QEvent *event)
                                 we->pixelDelta(), we->angleDelta(), we->delta(),
                                 we->orientation(), we->buttons(), we->modifiers(), we->phase());
 
-                QCoreApplication::sendEvent(this, &we2);
+                if (isVisible()) {
+                    QCoreApplication::sendEvent(this, &we2);
+                }
                 return true;
             }
             break;
@@ -1078,7 +1113,9 @@ bool Dialog::event(QEvent *event)
                     QDragEnterEvent de2(d->positionAdjustedForMainItem(de->pos()).toPoint(),
                                         de->possibleActions(), de->mimeData(), de->mouseButtons(), de->keyboardModifiers());
 
-                    QCoreApplication::sendEvent(this, &de2);
+                    if (isVisible()) {
+                        QCoreApplication::sendEvent(this, &de2);
+                    }
                     return true;
                 }
                 break;
@@ -1092,7 +1129,9 @@ bool Dialog::event(QEvent *event)
                     QDragMoveEvent de2(d->positionAdjustedForMainItem(de->pos()).toPoint(),
                                     de->possibleActions(), de->mimeData(), de->mouseButtons(), de->keyboardModifiers());
 
-                    QCoreApplication::sendEvent(this, &de2);
+                    if (isVisible()) {
+                        QCoreApplication::sendEvent(this, &de2);
+                    }
                     return true;
                 }
                 break;
@@ -1103,7 +1142,9 @@ bool Dialog::event(QEvent *event)
                     QDropEvent de2(d->positionAdjustedForMainItem(de->pos()).toPoint(),
                                 de->possibleActions(), de->mimeData(), de->mouseButtons(), de->keyboardModifiers());
 
-                    QCoreApplication::sendEvent(this, &de2);
+                    if (isVisible()) {
+                        QCoreApplication::sendEvent(this, &de2);
+                    }
                     return true;
                 }
                 break;
@@ -1182,6 +1223,9 @@ void Dialog::setVisible(bool visible)
 
     d->visible = visible;
     if (d->componentComplete) {
+        if (d->visualParent) {
+            setPosition(popupPosition(d->visualParent, size()));
+        }
         QQuickWindow::setVisible(visible);
         //signal will be emitted and proxied from the QQuickWindow code
     } else {
